
#property copyright "Coded by Andrew McKinnon 2016"
#property link      "http://penguintraders.com/members/andymack/"
#property version   "1.30"
#property strict


//+------------------------------------------------------------------+
//| Original source from FFCal
//| Most of the code is copied/stripped from FFCal                            
//| The result of the procedure call is a value between 0 and 3
//| -1 = no blocking of pair
//|  1 = blocking Low impact
//|  2 = blocking Medium impact
//|  3 = blocking High impact
//| 
//|  call the procedure        int getNewsImpactLevel( string pair,datetime refTime, int minutes before news, minutes after news);
//+------------------------------------------------------------------+
//| defines                                                          |
//+------------------------------------------------------------------+


                                                                        
/*---------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------------

ACKNOWLEDGEMENTS:

derkwehler and other contributors - the core code of the FFCal indicator,
                                    FFCal_v20 dated 07/07/2009, Copyright @ 2006 derkwehler
                                    http://www.forexfactory.com/showthread.php?t=19293
                                    email: derkwehler@gmail.com

deVries -      for his excellent donated work that significantly altered and streamlined the file
               handling coding to establish compatibility with the new release of MT4 Build 600+,
               (Jobs for deVries  www.mql5.com/en/job/new?prefered=deVries)
qFish -        for his generously given time and help during the effort to improve this indicator.
atstrader -    For the option controlling for what pair/pairs(s) news is shown, and for providing
               new file access coding in 2015.
traderathome - for the modification of the original work to show more and prioritized headlines.

---------------------------------------------------------------------------------------------------
Andrew McKinnon   Jan-2016
(Fuzzy)           http://penguintraders.com/members/andymack/
                  Thanks to Emonts (http://penguintraders.com/members/emonts/) 
                  
                  Changes made to facilitate being used as a library. 
                  Original Download routine kept, though, code changed to remove most of the 
                  global variables and compile warnings.
                  
                  New code should make it easier to expand the callable functions
                  
                  main callable functions
                  
                  int   getNewsImpactLevel   (string _symbol,datetime refTime, int minutesBefore, int minutesAfter)
                              
                              - returns the highest Impact level between minute ranges.  
                                I suggest  refTime = TimeGMT();
                  
                  int   getNewsItem          (structNewsItem& newsItem,string _symbol,datetime refTime,int minutesBefore,int minutesAfter)
                              
                              - returns index for the highest Impact level between minute ranges.                                
                              - newsItem (struct) is populated with the contents of the found news item. 
   
---------------------------------------------------------------------------------------------------
Andrew McKinnon   24-Jan-2016
(Fuzzy)           http://penguintraders.com/members/andymack/

                  -  Encapsulation into a Class
                  -  callable functions remain compatible
                  -  Enumeration of error codes added. 
                  -  Structure 'NewsCalendarItem' implemented.
                  -  only loads data if week has changed (ForexFactory files are week long) or
                     if a web download has occurred since last file load
                  -  use of MQL Globals for file and web locking.
                  -  File locks during read / write and checking routine will clear locks after 10 seconds
                     on assumtoption it must be wrong - no read / write should take that long. 
                  -  Web Locks during download.  separate routine checks on every access by user, 
                     if Lock exceeds 10 minutes it will be cleared.
                  -  Web access cycle time has nominal time eg., 240 minutes, is now varied by a 
                     Randomised 10 minute range.  Hopefully this will reduce ForexFactory Load.
                  03-Feb-2016   
                  -  Fixed bug by incorporating GetWinUtcDateTime to make sure the right time of Download
                  04-Feb-2016
                  -  Fixed bug (reloading file every time) with new truncateDate function
                  -  New log function using DebugView (microsoft)
---------------------------------------------------------------------------------------------------   
   
   
   
      USAGE   
      
      
       int   getNewsImpactLevel   (string _symbol,datetime refTime, int minutesBefore, int minutesAfter)
                              
                              - returns the highest Impact level between minute ranges.  
                                I suggest  refTime = TimeGMT();
                  
       int   getNewsItem          (structNewsItem& newsItem,string _symbol,datetime refTime,
                                    int minutesBefore,int minutesAfter)
                              
                              - returns index for the highest Impact level between minute ranges.                                
                              - newsItem (struct) is populated with the contents of the found news item.
      
        
   
                          
-------------------------------------------------------------------------------------------------*/







//#define READURL_BUFFER_SIZE   100

//#define DEBUGVIEW 1
#ifndef DEBUGVIEW
   #define DEBUGVIEW 0
#endif

#import  "Wininet.dll"
   int InternetOpenW(string, int, string, string, int);
   int InternetConnectW(int, string, int, string, string, int, int, int);
   int HttpOpenRequestW(int, string, string, int, string, int, string, int);

   int InternetOpenUrlW(int, string, string, uint, uint, uint);
   int InternetReadFile(int, uchar & arr[], int, int & arr[]);
   int InternetCloseHandle(int);
#import

#import "kernel32.dll"
int SystemTimeToFileTime(int& TimeArray[], int& FileTimeArray[]);
int FileTimeToLocalFileTime(int& FileTimeArray[], int& LocalFileTimeArray[]);
void GetSystemTime(int& TimeArray[]);

void OutputDebugStringW(string sMsg);
void OutputDebugStringA(uchar& szAnsiStr[]);
#import

enum  newsCalendarError
{
   newsCalendar_Null            =  0,
   newsItem_noneSelected        = -1,
   newsItems_Empty              = -2,
   newsCalFile_NotFound         = -12,
   newsCalFile_ReadError        = -13,
   newsCalFile_busy             = -14,
   newsCalFile_Error            = -15,
   urlAccess_NotFound           = -22,
   urlAccess_DownloadError      = -23,
   urlAccess_Busy               = -24,
   urlAccess_Error              = -25
};



struct      NewsCalendarItem
{
   string   Title ;
   string   Country; 
   int      countryId;
   datetime date;
   int      impactLevel;
   string   Actual;
   string   Forecast;
   string   Previous;
   int      Id;
};



//===========================================================================================+
//********************************************************************************************
//
//                                  Class Definition
//
//********************************************************************************************
//===========================================================================================+


class NewsCalendar
{
  
  
  
private :
uint READURL_BUFFER_SIZE ;
NewsCalendarItem newsCalItems[];    // where its all stored in array
string _currencies[]; 
string	     sUrl; 
int           WebUpdateFreq;        // downloads Calendar every xx minutes
int   lastFileChange;               // the datetime of last web DL. if != "UpdateHe.FF_Cal" array needs reloading 
int   minutesNextWebUpdate;         // minutes until next web updtae - gets randomized every time
datetime lastArray_CalendarDate;    // Calendar Date of current Array

   void initilize()
   {
      ArrayResize(_currencies,8);
      _currencies[0]="USD";_currencies[1]="EUR";_currencies[2]="GBP";_currencies[3]="JPY";
      _currencies[4]="AUD";_currencies[5]="CAD";_currencies[6]="CHF";_currencies[7]="NZD";
      minutesNextWebUpdate =  randomWebUpdateMinutes(WebUpdateFreq*60);
      WebUpdateFreq = 240;
      READURL_BUFFER_SIZE = 100;
      lastFileChange = int(GlobalVariableGet("UpdateHe.FF_Cal"));
   }

public   :
   NewsCalendar()
   {
      sUrl = "http://www.forexfactory.com/ff_calendar_thisweek.xml";
      initilize();
   }
   NewsCalendar(string url)
   {
      sUrl = url;
      initilize();
   }
NewsCalendar(string url,int minutesUrlUpdate)
   {
      sUrl = url;
      initilize();
      WebUpdateFreq = minutesUrlUpdate;
   }

   void deInit()
   {
      ArrayResize(newsCalItems,0);
      ArrayResize(_currencies,0);
   }
   
//+-----------------------------------------------------------------------------------------------+
//| user function:                        getNewsImpactLevel                                      |
//+-----------------------------------------------------------------------------------------------+

//     - returns the highest Impact level between minute ranges for given curr pair.
//     - typically, refTime  = TimeGMT();
//     - returns error as per ** newsCalendarError **

int getNewsImpactLevel(string _symbol,datetime refTime, int minutesBefore, int minutesAfter)
{
   
   string base, quote;
   bool block = false;
   string savedImpact = "";
   string __symbol = parseSymbolName(_symbol);
   
   
   base = StringSubstr(__symbol,0,3);
   quote = StringSubstr(__symbol,3,3);
   int curr1 = countryID(base);
   int curr2 = countryID(quote);
   int minutes_TillNews;
   int err;
   
   err=refreshNewsCalendar(sUrl);
   if (err<0)return(err);
   //datetime calendardate = calendarDate(refTime);
   //string xmlFileName = GetXmlFileName(calendardate);
   //int rows = fillNewsItemsArray(xmlFileName,newsCalItems);
   int rows = freshenNewsItemArray(refTime);
   if (rows < 1) return (rows); // rows = errorcode
   int impact = newsItem_noneSelected;
   for (int i=0;i<rows;i++)
   {
      if (newsCalItems[i].countryId != curr1 && newsCalItems[i].countryId != curr2) continue;
      minutes_TillNews = (int(newsCalItems[i].date) - int(refTime)) / 60;
      if (minutesAfter + minutes_TillNews > 0 )
      {
         if (minutes_TillNews < minutesBefore)
         {
            if (newsCalItems[i].impactLevel  > impact) impact = newsCalItems[i].impactLevel;
         }
      
      }
   }
   
   return (impact);
}

//+-----------------------------------------------------------------------------------------------+
//| user function:                           getNewsItem                                          |
//+-----------------------------------------------------------------------------------------------+

//       - returns pointer to newsItems struct array for item of highest Impact Level 
//                                         between minute ranges for given curr pair.
//       - populates newsItem from the found element in newsItems array
//       - returns error as per ** newsCalendarError **


int   getNewsItem(NewsCalendarItem& newsItem, 
                  string _symbol,
                  datetime refTime, 
                  int minutesBefore, 
                  int minutesAfter)
{
   NewsCalendarItem newsError;
   string base, quote;
   bool block = false;
   string savedImpact = "";
   string __symbol = parseSymbolName(_symbol);
   int ptr = -1;
   
   newsItem.Country = "";
   newsItem.countryId = -1;
   newsItem.Title = "";
   newsItem.date = 0;
   newsItem.impactLevel = -1;
   newsItem.Forecast = "";
   newsItem.Previous = "";
   
   newsError.Title = "Error";
   base = StringSubstr(__symbol,0,3);
   quote = StringSubstr(__symbol,3,3);
   int curr1 = countryID(base);
   int curr2 = countryID(quote);
   int minutes_TillNews;
   
   int err = refreshNewsCalendar(sUrl);
   if (err<0)return(err);
   
   int rows = freshenNewsItemArray(refTime);
   //datetime calendardate = calendarDate(refTime);
   //string xmlFileName = GetXmlFileName(calendardate);
   //int rows = fillNewsItemsArray(xmlFileName,newsCalItems);
   if (rows < 1) return (rows);
   int impact = -1;
   for (int i=0;i<rows;i++)
   {
      if (newsCalItems[i].countryId != curr1 && newsCalItems[i].countryId != curr2) continue;
      minutes_TillNews = (int(newsCalItems[i].date) - int(refTime)) / 60;
      if (minutesAfter + minutes_TillNews > 0 )
      {
         if (minutes_TillNews < minutesBefore)
         {
            if (newsCalItems[i].impactLevel  > impact) {impact = newsCalItems[i].impactLevel;ptr=i;}
         }
      }
   }
   if (ptr < 0) return(newsItem_noneSelected);
   newsItem.Country = newsCalItems[ptr].Country;
   newsItem.countryId = newsCalItems[ptr].countryId;
   newsItem.Title = newsCalItems[ptr].Title;
   newsItem.date = newsCalItems[ptr].date;
   newsItem.impactLevel = newsCalItems[ptr].impactLevel;
   newsItem.Actual = newsCalItems[ptr].Actual;
   newsItem.Forecast = newsCalItems[ptr].Forecast;
   newsItem.Previous = newsCalItems[ptr].Previous;
   newsItem.Id = newsCalItems[ptr].Id;
   return (ptr);
}



//+===============================================================================================+
//
//
//            ******** END *******    USER Functions
//
//                                    
//+===============================================================================================+


/**
         the GetWin Local and UTC time routines source from
         https://forum.mql4.com/9900#400075
         brewmanz
         
         Public declared:  available to indicators and EA
*/
  

datetime GetWinUtcDateTime()
{
   double hundrednSecPerSec = 10.0 * 1000000.0;
   double bit32to64 = 65536.0 * 65536.0;
   double secondsBetween1601And1970 = 11644473600.0;
   int    TimeArray[4];
   int    FileTimeArray[2];   // 100nSec since 1601/01/01 UTC
   GetSystemTime(TimeArray);
   SystemTimeToFileTime(TimeArray, FileTimeArray);
   
   double lfLo32 = FileTimeArray[0];
   if(lfLo32 < 0)
      lfLo32 = bit32to64 + lfLo32;
   double ticksSince1601 = FileTimeArray[1] * bit32to64 + lfLo32;
   double secondsSince1601 = ticksSince1601 / hundrednSecPerSec;
   double secondsSince1970 = secondsSince1601 - secondsBetween1601And1970;
   // McKinnon:   repair data conversion warning
   datetime dt = int(secondsSince1970);
   return (dt);
}

datetime GetWinLocalDateTime()
{
   double hundrednSecPerSec = 10.0 * 1000000.0;
   double bit32to64 = 65536.0 * 65536.0;
   double secondsBetween1601And1970 = 11644473600.0;
   int    TimeArray[4];
   int    FileTimeArray[2];   // 100nSec since 1601/01/01 UTC
   int    LocalFileTimeArray[2];   // 100nSec since 1601/01/01 Local

   GetSystemTime(TimeArray);
   SystemTimeToFileTime(TimeArray, FileTimeArray);
   FileTimeToLocalFileTime(FileTimeArray, LocalFileTimeArray);
   
   double lfLo32 = LocalFileTimeArray[0];
   if(lfLo32 < 0)
      lfLo32 = bit32to64 + lfLo32;
   double ticksSince1601 = LocalFileTimeArray[1] * bit32to64 + lfLo32;
   double secondsSince1601 = ticksSince1601 / hundrednSecPerSec;
   double secondsSince1970 = secondsSince1601 - secondsBetween1601And1970;
   // McKinnon:   repair data conversion warning
   datetime dt = int(secondsSince1970);
   return (dt);
}
private :






//+-----------------------------------------------------------------------------------------------+
//| Subroutines: Refreshes Current Calendar and checks WebLock                                    |
//+-----------------------------------------------------------------------------------------------+  

//deVries: void InitNews(string& mainData[][], string newsUrl)
// McKinnon:   now returns an error condition
//             - renamed subroutine to more fitting name
//             - Should be run once from every User Function
int refreshNewsCalendar(string newsUrl)
   {
   int err = 0;
   if (IsTesting()) return (0);
   if (IsOptimization()) return (0);
   checkWebLock();      //    checks that web Lock hasn't expired
   if(DoFileDownLoad()) //Added to check if the CSV file already exists
      {
         err = DownLoadWebPageToFile(newsUrl); //downloading the xml file
      }
   return(err);
   }


   
//deVries: If we have recent file don't download again
//    McKinnon:   no need to load file
//                truncated to remove file load
bool DoFileDownLoad()
   {
   int xmlHandle = 0;
   //datetime time = TimeCurrent();
   datetime time = GetWinUtcDateTime();
   //Print("UTC = " + TimeToString(time,TIME_DATE|TIME_MINUTES));
   if(GlobalVariableCheck("UpdateHe.FF_Cal") == false)return(true);
   if((time - GlobalVariableGet("UpdateHe.FF_Cal")) > minutesNextWebUpdate)return(true);
   
   return (false);
   }
 
//+-----------------------------------------------------------------------------------------------+
//| Subroutine: downloading the ForexFactory .xml file                                            |
//+-----------------------------------------------------------------------------------------------+


//2014 deVries: new coding replacing old "GrabWeb" coding
//2015 atstrader: revise file access coding
//2106   McKinnon:   implementing file sharing and web access sharing 
//                   -  save to tmp file to reduce file sharing timeing issues.
//                   -  web sharing
//                   -  cleaned up some of the logic
int DownLoadWebPageToFile(string url = "http://www.forexfactory.com/ff_calendar_thisweek.xml")
   {
   //int response = 0;
   int HttpOpen = InternetOpenW(" ", 0, " ", " ", 0);
   int HttpConnect = InternetConnectW(HttpOpen, "", 80, "", "", 3, 0, 1);
   
   bool ready = WebAccess_Ready();   // someone else is accessing the Web News Cal
   if (!ready) return (urlAccess_Busy);
   
   GlobalVariableSet("NewsCal_WB",1.0);      // its mine, (Web sharing)
   
//start 2015 atstrader revision
   // int HttpRequest = InternetOpenUrlW(HttpOpen, url, NULL, 0, 0, 0);
   // INTERNET_FLAG_PRAGMA_NOCACHE - 0x00000100 - do not try the cache or proxy
   // INTERNET_FLAG_NO_CACHE_WRITE - 0x04000000 - don't add this to the IE cache
   // INTERNET_FLAG_RELOAD         - 0x80000000 - Forces download of requested file, object, or
   //                                directory listing from the origin server, not from the cache.
   int HttpRequest = InternetOpenUrlW(HttpOpen, url, NULL, 0, 0x84000100, 0);
//end 2015 atstrader revision
   if (HttpRequest == -1  ) return(urlAccess_NotFound);
   int read[1];
   uchar  Buffer[];
   ArrayResize(Buffer, READURL_BUFFER_SIZE + 1);
   string NEWS = "";
   /*
      datetime calendardate = calendarDate(TimeCurrent());
	   string xmlFileName = GetXmlFileName(calendardate) + ".tmp";  //      tmp
	   int xmlHandle = FileOpen(xmlFileName, FILE_BIN|FILE_READ|FILE_WRITE);
	   //File exists if FileOpen return >=0.
	   if (xmlHandle >= 0) {FileClose(xmlHandle); FileDelete(xmlFileName);}

		//Open new XML.  Write the ForexFactory page contents to a .htm file.  Close new XML.
		xmlHandle = FileOpen(xmlFileName, FILE_BIN|FILE_WRITE);
   */
   while (true)
      {
      InternetReadFile(HttpRequest, Buffer, READURL_BUFFER_SIZE, read);
      string strThisRead = CharArrayToString(Buffer, 0, read[0], CP_UTF8);
      if (read[0] > 0)NEWS = NEWS + strThisRead;
      else
         {
         
         //FileWriteString(xmlHandle, NEWS);
         //FileClose(xmlHandle);
		   //Find the XML end tag to ensure a complete page was downloaded.
		   int end = StringFind(NEWS, "</weeklyevents>", 0);
		   //If the end of file tag is not found, a return -1 (or, "end <=0" in this case),
		   //then return (false).
		   if (end == -1)
		      {
		      Alert(Symbol()," ",Period(),", FFCal Error: File download incomplete!");
		      GlobalVariableSet("NewsCal_WB",0.0);      // release the lock, (Web sharing)
		      return(urlAccess_DownloadError);
		      }
         break;
         }
      }
   int err = writeCurrentCalendarFile(NEWS);   
   // McKinnon :  changed time to Win UTC
   GlobalVariableSet("UpdateHe.FF_Cal", GetWinUtcDateTime());
   minutesNextWebUpdate =  randomWebUpdateMinutes(WebUpdateFreq*60);          // set a randomised update time (600 sec random range)
   if (HttpRequest > 0) InternetCloseHandle(HttpRequest);
   if (HttpConnect > 0) InternetCloseHandle(HttpConnect);
   if (HttpOpen > 0) InternetCloseHandle(HttpOpen);
   GlobalVariableSet("NewsCal_WB",0.0);      // release the lock, (Web sharing)
   return (err);
   }


//+-----------------------------------------------------------------------------------------------+
//| Subroutine: return the Date of the Sunday preceeding given date  (used in FF xml filename)    |
//+-----------------------------------------------------------------------------------------------+

//McKinnon: new function calendardate 
//          extracted from old GetXMLFileName 

datetime calendarDate(datetime currentDate)
{
   int adjustDays = 0;
   switch(TimeDayOfWeek(currentDate))
      {
      case 0:
      adjustDays = 0;
      break;
      case 1:
      adjustDays = 1;
      break;
      case 2:
      adjustDays = 2;
      break;
      case 3:
      adjustDays = 3;
      break;
      case 4:
      adjustDays = 4;
      break;
      case 5:
      adjustDays = 5;
      break;
      case 6:
      adjustDays = 6;
      break;
      }
   datetime calendardate =  currentDate - (adjustDays  * 86400);
   calendardate = truncateDateTime(calendardate);
   return (calendardate);//Always a Sunday date
}

 
//+-----------------------------------------------------------------------------------------------+
//| Subroutine: to pad string                                                                     |
//+-----------------------------------------------------------------------------------------------+
//deVries:
string PadString(string toBePadded, string paddingChar, int paddingLength)
   {
   while(StringLen(toBePadded) <  paddingLength)
      {
      toBePadded = StringConcatenate(paddingChar,toBePadded);
      }
   return (toBePadded);
   }
   


//+-----------------------------------------------------------------------------------------------+
//| Indicator Subroutine For Date/Time    changed by deVries                                      |
//+-----------------------------------------------------------------------------------------------+
datetime MakeDateTime(string strDate, string strTime)  //not string now datetime
   {
	int n1stDash = StringFind(strDate, "-");
	int n2ndDash = StringFind(strDate, "-", n1stDash+1);

	string strMonth = StringSubstr(strDate, 0, 2);
	string strDay = StringSubstr(strDate, 3, 2);
	string strYear = StringSubstr(strDate, 6, 4);

	int nTimeColonPos = StringFind(strTime, ":");
	string strHour = StringSubstr(strTime, 0, nTimeColonPos);
	string strMinute = StringSubstr(strTime, nTimeColonPos+1, 2);
	string strAM_PM = StringSubstr(strTime, StringLen(strTime)-2);

	int nHour24 = StrToInteger(strHour);
	if ((strAM_PM == "pm" || strAM_PM == "PM") && nHour24 != 12) {nHour24 += 12;}
	if ((strAM_PM == "am" || strAM_PM == "AM") && nHour24 == 12) {nHour24 = 0;}

   // McKinnon:  fix type conversions
   datetime newsevent = StringToTime(strYear+ "." + strMonth + "." + strDay) +
                         nHour24*3600 + StrToInteger(strMinute)*60 ;
   
	return(newsevent);
   }
   

    
//+-----------------------------------------------------------------------------------------------+
//| Subroutines by Andrew McKinnon                                                                |
//+-----------------------------------------------------------------------------------------------+  
   
 
 
//+-----------------------------------------------------------------------------------------------+
//| Subroutine:                        strips unwanted prefix from symbom                         |
//+-----------------------------------------------------------------------------------------------+
  
   
string parseSymbolName(string symBol)
{
   string retSymbol=symBol;
   if (StringLen(symBol) > 6)
   {
      string firstPart = StringSubstr(symBol, 0, 3);
      bool found = false;
      for (int i = 0; i < ArraySize(_currencies); i++)
      {
         if (firstPart == _currencies[i]) found = true;
      }
      if (found)
      {
         retSymbol = StringSubstr(symBol,0,6);
      }
      else
      {
        retSymbol = StringSubstr(symBol,StringLen(symBol) - 6, 6);
      } 
   }
   return (retSymbol);
}

//+-----------------------------------------------------------------------------------------------+
//| Subroutines:     Freshen News Item Array  ** if it needs                                      |
//+-----------------------------------------------------------------------------------------------+  
   
   int   freshenNewsItemArray(datetime _time)
   {
      int size = ArraySize(newsCalItems);
      bool loadFile = false;
      datetime ftime = calendarDate(_time);
      datetime _recent = calendarDate(GetWinUtcDateTime());
      //                         load because it needs back data
      if ( lastArray_CalendarDate != ftime) loadFile = true;  
      // reload if current data has a fresh download
      log(__FUNCTION__, IntegerToString(int(loadFile)));                        
      if ( _recent == ftime  && 
            lastFileChange != int( GlobalVariableGet("UpdateHe.FF_Cal")))  loadFile = true ;
      
      if (loadFile)
      {
         string xmlFileName = GetXmlFileName(ftime);
         log("Loading xmlFile: " , xmlFileName);
         size = fillNewsItemsArray(xmlFileName,newsCalItems);
         lastFileChange = int( GlobalVariableGet("UpdateHe.FF_Cal"));   // sets it s fresh
         log("times= ",TimeToStr(lastArray_CalendarDate,TIME_DATE|TIME_MINUTES|TIME_SECONDS),TimeToStr(ftime,TIME_DATE|TIME_MINUTES|TIME_SECONDS));
         lastArray_CalendarDate = ftime;                                // sets date of array
         log("timess= ",TimeToStr(lastArray_CalendarDate,TIME_DATE|TIME_MINUTES|TIME_SECONDS),TimeToStr(ftime,TIME_DATE|TIME_MINUTES|TIME_SECONDS));
      }
      return(size);
   }

//+-----------------------------------------------------------------------------------------------+
//| Subroutine:         fill "the" newsCalItems array  from FILE                                            |
//+-----------------------------------------------------------------------------------------------+


int  fillNewsItemsArray(string newsFile, NewsCalendarItem& _NewsItems[])
{
   int rows = 0;
   string buffer = "";
   string sEvents[];
   
   
   ArrayResize(_NewsItems,0);
   buffer = return_buffer_fromXMLfile(newsFile);
   if (StringLen(buffer) < 1) return(newsCalFile_NotFound);
   rows = parse_Buffer_into_Events(buffer,sEvents);
   if (rows < 1) return (newsItems_Empty);
   ArrayResize(_NewsItems,rows);
   for (int i=0;i<rows;i++)
   {
      int flds = parse_Event(sEvents[i],_NewsItems[i]);
   }
   return(rows);
}

//+-----------------------------------------------------------------------------------------------+
//| function:                  Read Calendar File into Buffer                                     |
//+-----------------------------------------------------------------------------------------------+ 

string return_buffer_fromXMLfile(string xmlFile)
{
   //InitNews(sUrl);
   string buffer;
   //ArrayResize(NewsItems,0);
   
   bool ready = calenderFile_Ready();    //    file sharing
   if (!ready) 
   {
      //Print("NewsCalendarFile failed Busy : " + xmlFile);
      return ("");
   }
   //Print(" FILE tryying to open " + xmlFile);
   GlobalVariableSet("NewsCal_FB",1.0);         //              Lock the file
   int _handle = FileOpen(xmlFile, FILE_BIN|FILE_READ);
   if (_handle == INVALID_HANDLE)
   {
      //Print("NewsCalendarFile failed to open : " + xmlFile);
      GlobalVariableSet("NewsCal_FB",0.0);      //             unLock the file
      return ("");
   }
	if(_handle>=0)
	{
	   int size = int(FileSize(_handle));
	   buffer = FileReadString(_handle, size);
	   FileClose(_handle);
	}
	GlobalVariableSet("NewsCal_FB",0.0);          //             unLock the file
	return(buffer);
}


//+-----------------------------------------------------------------------------------------------+
//| Subroutines: clears the Web Lock if something bad happens (10 minutes)                            |
//+-----------------------------------------------------------------------------------------------+  

void  checkWebLock()
{
   static datetime last_UnlockTime=0;// GetWinUtcDateTime();
   
   if (int(last_UnlockTime) == 0) last_UnlockTime = GetWinUtcDateTime();
   int busy = int( GlobalVariableGet("NewsCal_WB"));
   if (busy == 0) last_UnlockTime = GetWinUtcDateTime();
   if (int(GetWinUtcDateTime() - int(last_UnlockTime)) > 600) 
   {
      GlobalVariableSet("NewsCal_WB",0.0);      // clear the lock 
   }
   
}



//+-----------------------------------------------------------------------------------------------+
//| Subroutine: Web access sharing.  returns TRUE if ready , doesn't clear Locks                  |
//+-----------------------------------------------------------------------------------------------+

bool     WebAccess_Ready()
{
   //    no need for multiple retries 
   int busy = true;
   MathSrand(GetTickCount());
   int wait = MathRand()%10 + 1; 
   Sleep(wait);         //       random wait up to 10mSec to stop races 
   busy = int( GlobalVariableGet("NewsCal_WB"));
   if (busy == 0) return(true);
   return(false);
}
 

//+-----------------------------------------------------------------------------------------------+
//| Subroutine:                           File Sharing                                            |
//+-----------------------------------------------------------------------------------------------+ 

//    McKinnon:   file sharing function
//                   - NewCal_FB ,(File Busy) 
//                     is Busy if NewCal_FB = 1; otherwise 0
//                   - Tries 10 times then assumes other APP left it busy, so clears it  
bool     calenderFile_Ready()
{
   int busy ;
   MathSrand(GetTickCount());
   int wait = MathRand()%10 + 1; 
   Sleep(wait);         //       random wait up to 10mSec to stop races 
   if(GlobalVariableCheck("NewsCal_FB") == false) return true;     
   for (int retries = 0;retries < 10;retries ++)
   {
      busy = int( GlobalVariableGet("NewsCal_FB") );
      if (busy == 0) break;
      Sleep (1000);
   }  
   GlobalVariableSet("NewsCal_FB",0.0);      //    Clear it anyway
   return(true);
}
 
//+-----------------------------------------------------------------------------------------------+
//| Subroutine:      return string array from file buffer                                         |
//+-----------------------------------------------------------------------------------------------+


int parse_Buffer_into_Events(string sBuffer,string& strEvents[] )
{
   int ptr=0;
   int ptrEnd;
   int rows = 0;
   
   
   while(true)
   {
      ptr = StringFind(sBuffer, "<event>", ptr);
      if (ptr < 0) break;
      ptr += 7;
      ptrEnd  = StringFind(sBuffer, "</event>", ptr);
      if (ptrEnd < 0) break;
      string sEvent="";
      sEvent = StringSubstr(sBuffer, ptr, ptrEnd - ptr);
      rows ++;
      ArrayResize(strEvents,rows);
      strEvents[rows-1] = sEvent;
   }
   return (rows);
}

//+-----------------------------------------------------------------------------------------------+
//| Subroutine:      parses data from a line of XML data string                                   |
//+-----------------------------------------------------------------------------------------------+


int parse_Event(string strEvent,NewsCalendarItem& NewsItem)
{
   string tags[] = {"title","country","date","time","impact","forecast","previous"};
   string sdata[];
   int ptr,ptrEnd;
   int fields = 0;
   int tagLen;
   string startTag,endTag,strField;
   
   int size = ArraySize(tags);
   ArrayResize(sdata,size);
   for (int i=0;i<size;i++)
   {
      startTag = "<" + tags[i] + ">";
      endTag = "</" + tags[i] + ">";
      tagLen = StringLen(startTag);
      sdata[i]="";
      ptr = StringFind(strEvent, startTag, 0);
      ptrEnd  = StringFind(strEvent, endTag, ptr+tagLen);
      if (ptr > -1 && ptrEnd > ptr)
      { 
         strField = StringSubstr(strEvent, ptr + tagLen, ptrEnd - ptr-tagLen);
         sdata[i] = cleanXMLdata(strField);
         fields ++;
      }
   }
   
   NewsItem.Title =sdata[0];
   NewsItem.Country = sdata[1];
   NewsItem.countryId = countryID(NewsItem.Country);
   NewsItem.Forecast =sdata[5];
   NewsItem.Previous =sdata[6];
   NewsItem.date =  MakeDateTime(sdata[2],sdata[3]);  
   NewsItem.impactLevel = 0;
   if (StringCompare(sdata[4],"Low",false)==0) NewsItem.impactLevel = 1;
   if (StringCompare(sdata[4],"Medium",false)==0) NewsItem.impactLevel = 2;
   if (StringCompare(sdata[4],"High",false)==0) NewsItem.impactLevel = 3;
   return (fields);
}

//+-----------------------------------------------------------------------------------------------+
//| Subroutine:      Strips unwanted chars from XML string                                        |
//+-----------------------------------------------------------------------------------------------+


string cleanXMLdata(string Xmldata)
{
   string clean = StringTrimLeft(Xmldata);
   clean  = StringTrimRight(clean);
   StringReplace(clean,"![CDATA[","");
   StringReplace(clean,"]","");
   StringReplace(clean,"[","");
   StringReplace(clean,"<","");
   StringReplace(clean,">","");
   return(clean);
}


//+-----------------------------------------------------------------------------------------------+
//| Subroutine: Writes buffer to Current Date Calendar file                                       |
//+-----------------------------------------------------------------------------------------------+

//    McKinnon:   writes the buffer (from web download) into current file
//                - pulled this out of Web download routine to make file locking 
//                  duration short enough to allow for the file sharing method 
int writeCurrentCalendarFile(string buffer)
{
   bool ready = calenderFile_Ready();    //    file sharing
   if (!ready) return(newsCalFile_busy);
   
   GlobalVariableSet("NewsCal_FB",1.0);         //              Lock the file   
   // McKinnon :  Get Real UTC time
   datetime calendardate = calendarDate(GetWinUtcDateTime());
	string xmlFileName = GetXmlFileName(calendardate);
	int xmlHandle = FileOpen(xmlFileName, FILE_BIN|FILE_READ|FILE_WRITE);
	//File exists if FileOpen return >=0.
	if (xmlHandle >= 0) {FileClose(xmlHandle); FileDelete(xmlFileName);}

		//Open new XML.  Write the ForexFactory page contents to a .htm file.  Close new XML.
   xmlHandle = FileOpen(xmlFileName, FILE_BIN|FILE_WRITE);
   uint len = FileWriteString(xmlHandle, buffer);
   FileClose(xmlHandle);
   GlobalVariableSet("NewsCal_FB",0.0);         //              unLock the file  
   if (len < 1) return (newsCalFile_Error);
   return (int(len));
   
}



//+-----------------------------------------------------------------------------------------------+
//| Subroutine: return the name of the ForexFactory .xml file                                    |
//+-----------------------------------------------------------------------------------------------+
//
//       McKinnon:   GetXMLFileName now takes a Date parameter
//                         - (allows for loading past files)
//                         - use  calendarDate to get suitable sunday date.
//                         - change Filename format.  to  FFCal_2016-01-17.xml
//                               much easier to find stored files 

string GetXmlFileName(datetime calendardate)
   {
   /*
   string fileName =  (StringConcatenate(TimeYear(calendardate),"-",
          PadString(DoubleToStr(TimeMonth(calendardate),0),"0",2),"-",
          PadString(DoubleToStr(TimeDay(calendardate),0),"0",2),"-FFCal-News",".xml"));
*/
   string filename = StringConcatenate("FFCal_",
                                        TimeYear(calendardate),"-",
                                        PadString(DoubleToStr(TimeMonth(calendardate),0),"0",2),"-",
                                        PadString(DoubleToStr(TimeDay(calendardate),0),"0",2),".xml"
                                        );
                                          
   return (filename); //Always a Sunday date
   }
  
  
//+-----------------------------------------------------------------------------------------------+
//| Subroutines:     Country ID                                                                   |
//+-----------------------------------------------------------------------------------------------+  
   

int countryID(string strCountry)
{
   for (int i = 0; i < ArraySize(_currencies)-1; i++)
   {
      if (StringCompare(strCountry,_currencies[i]) == 0) return (i+1);
   }
   return (-1);
}

//+-----------------------------------------------------------------------------------------------+
//| Subroutines:     Randomly changes Web download Time                                           |
//+-----------------------------------------------------------------------------------------------+  
   


int   randomWebUpdateMinutes(int updateMin)
{
   MathSrand(GetTickCount());
   int wait = int(MathRand()/50) ; 
   return(updateMin+wait);
}


//+-----------------------------------------------------------------------------------------------+
//| Subroutines:     whether Current File has been Refreshed                                      |
//+-----------------------------------------------------------------------------------------------+  
 

   bool hasCurrentFileRefreshed()
   {
      int _lfc = int(GlobalVariableGet("UpdateHe.FF_Cal"));
      if(lastFileChange != _lfc) return (true);
      return(false);
   }
   
//+-----------------------------------------------------------------------------------------------+
//| Subroutines:     Truncates DateTime                                                           |
//+-----------------------------------------------------------------------------------------------+  

//    @truncate of value 3 will prune hours,minute,seconds off the date
  
   datetime truncateDateTime(datetime time,int truncate = 3)
   {
      MqlDateTime d;
      TimeToStruct(time,d);
      switch (truncate)
      {
         case 5:  d.mon =  1;
         case 4:  d.day =  1;
         case 3:  d.hour = 0;
         case 2:  d.min =  0;
         case 1:  d.sec =  0;
         case 0:  break;
      }
      return(StructToTime(d));
   }

//+-----------------------------------------------------------------------------------------------+
//| Subroutines:     Log to DEBUGVIEW                                                             |
//+-----------------------------------------------------------------------------------------------+  

//   http://www.forexfactory.com/showthread.php?t=245303
public:
   void log(
   string s1, 
   string s2="", 
   string s3="", 
   string s4="", 
   string s5="", 
   string s6="", 
   string s7="", 
   string s8=""
   ){
   string out = StringTrimRight(StringConcatenate(
      WindowExpertName(), ".mq4 ", Symbol(), 
      " ", s1, 
      " ", s2, 
      " ", s3, 
      " ", s4, 
      " ", s5, 
      " ", s6, 
      " ", s7, 
      " ", s8
   ));
   if (DEBUGVIEW == 1) OutputDebugStringW(out);
}

};

/******************************************************************************************************************/


//                                           END of CLASS definition


/******************************************************************************************************************/

