//+------------------------------------------------------------------+
//|                                        TradingRoutines_V1.01.mqh |
//|                        Copyright 2016, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property strict
//+------------------------------------------------------------------+
//| defines                                                          |
//+------------------------------------------------------------------+
// #define MacrosHello   "Hello, world!"
// #define MacrosYear    2010
//+------------------------------------------------------------------+
//| DLL imports                                                      |
//+------------------------------------------------------------------+
// #import "user32.dll"
//   int      SendMessageA(int hWnd,int Msg,int wParam,int lParam);
// #import "my_expert.dll"
//   int      ExpertRecalculate(int wParam,int lParam);
// #import
//+------------------------------------------------------------------+
//| EX5 imports                                                      |
//+------------------------------------------------------------------+
// #import "stdlib.ex5"
//   string ErrorDescription(int error_code);
// #import
//+------------------------------------------------------------------+

#include <stdlib.mqh>
#include <WinUser32.mqh>

// Constants
#define SYMBOL_ALL                 "-1"   // all symbols
#define OP_ALL                      -1    // all order types
#define CLR_SELL                    clrRed
#define CLR_BUY                     clrSkyBlue
#define CLR_MODIFY                  clrOrange
#define NO_ERROR                    0
#define STD_ERROR                   -1
#define BUY_ORDERS                  1
#define SELL_ORDERS                 2
#define ALL_ORDERS                  3

class TradingRoutines
{


private:
//////////////////////////////////////////////////////////////////////////////////////////////////
double      _myMultiplier;
double      _myPoint;
double      _myDigits;

string stringToLog;
string fileName;

void initialize()
{
   _myDigits = MarketInfo(Symbol(), MODE_DIGITS);
   _myPoint = MarketInfo(Symbol(), MODE_POINT);
   if (_myDigits == 5 || _myDigits == 3 || _myDigits == 1 ) { _myPoint *= 10.0;}
	if (_myDigits == 2  ) {_myPoint *= 100.0;}
	_myMultiplier = 1.0 / _myPoint;
}

/* -----------------------------------------------------------
   waitForTradeContext
	Author:		simplex, 2016
	Returns: 	false if trade context is occupied
					true if trade context is free
   ----------------------------------------------------------- */
bool waitForTradeContext(const string message) {
	while (IsTradeContextBusy()) {
		Print(message + "Waiting for trade context.");
		Sleep(MathRand() / 10);
	}
	return(true);
}

/* -------------------------------------------------------------------
	eaLog
	Author:		simplex, 2016
	Returns: 	formatted string to print out in a log file
	------------------------------------------------------------------- */
string eaLog(string _input1, string _input2, int tabSize = 30) {
	_input1 = StringLeftTrim(StringRightTrim(_input1)) + " ";
	_input2 = StringLeftTrim(StringRightTrim(_input2));
	while(StringLen(_input1) < tabSize) {
		_input1 = _input1 + " ";
	}
	string out = _input1 + _input2;
	if (StringLen(_input2) > 0) {
		out = out + " ";
	}
	return(out);
}

/* -------------------------------------------------------------------
	price2String
	Author:		simplex, 2016
	Returns: 	formatted price string to print out in a log file
	Remarks:		depends on global variable digits
	------------------------------------------------------------------- */
string price2String(double _input) {
	string out = DoubleToStr(_input, _myDigits);
	return(out);
}


/* -------------------------------------------------------------------
	ticket2String
	Author:		simplex, 2016
	Returns: 	formatted ticket string to print out in a log file
	------------------------------------------------------------------- */
string ticket2String(int ticket) {
	string out = "Ticket_" + IntegerToString(ticket) + " ";
	return(out);
}

/* -------------------------------------------------------------------
	IsTemporaryError
	Is the error temporary (does it make sense to wait).
	------------------------------------------------------------------- */
bool IsTemporaryError(int error) {
   return(error == ERR_NO_ERROR ||
          error == ERR_COMMON_ERROR ||
          error == ERR_SERVER_BUSY ||
          error == ERR_NO_CONNECTION ||
          error == ERR_MARKET_CLOSED ||
          error == ERR_PRICE_CHANGED ||
          error == ERR_INVALID_PRICE ||  //happens sometimes
          error == ERR_OFF_QUOTES ||
          error == ERR_BROKER_BUSY ||
          error == ERR_REQUOTE ||
          error == ERR_TRADE_TIMEOUT ||
          error == ERR_TRADE_CONTEXT_BUSY );
}


public:
//////////////////////////////////////////////////////////////////////////////////////////////////

TradingRoutines()
{
   initialize();
}

/* -------------------------------------------------------------------
	OrderSendReliable
	ordersend reliable
	simplex: modified logging
	------------------------------------------------------------------- */
int OrderSendReliable(string symbol,
                      int cmd,
                      double volume,
                      double price,
                      int _slippage,
                      double stoploss,
                      double takeprofit,
                      int _magic,
                      string comment = "",
                      datetime expiration = 0,
                      color arrow_color = clrNONE
							) {
   int ticket;
   
   int err;
   //int __magic =0;
	string printPrefix = eaLog(__FUNCTION__, "");
	
	stringToLog = printPrefix
         + OrderTypeToStr(cmd) + ", "
         + symbol + ", "
         + DoubleToStr(volume, 2) + ", "
         + price2String(price) + ", "
         + IntToStr(_slippage) + ", "
         + price2String(stoploss) + ", "
         + price2String(takeprofit) + ", "
         + comment + ", "
         + IntToStr(_magic) + ", "
         + TimeToStr(expiration) + ", "
         + ColorToString(arrow_color) + ")";
         
   Print(  stringToLog);

   while (true) {
      if (IsStopped()) {
         Print(printPrefix + " Trading is stopped!");
         return(-1);
      }
      RefreshRates();
      if (cmd == OP_BUY) {
         price = MarketInfo(symbol, MODE_ASK);
         //__magic = magicNumberBuy;
      }
      if (cmd == OP_SELL) {
         price = MarketInfo(symbol, MODE_BID);
         //__magic = magicNumberSell;
      }
      if (!IsTradeContextBusy()) {
         ticket = OrderSend(symbol,
                            cmd,
                            volume,
                            NormalizeDouble(price, (int)MarketInfo(symbol, MODE_DIGITS)),
                            _slippage,
                            NormalizeDouble(stoploss, (int)MarketInfo(symbol, MODE_DIGITS)),
                            NormalizeDouble(takeprofit, (int)MarketInfo(symbol, MODE_DIGITS)),
                            comment,
                            _magic,
                            expiration,
                            arrow_color);
         if (ticket > 0) {
				printPrefix = printPrefix + ticket2String(ticket);
            Print(printPrefix + "opened.");
            
            // log trade info
            writeToLogFile(stringToLog, "STO_");
            return(ticket); 						// normal exit
         }

         err = GetLastError();
         if (IsTemporaryError(err)) {
            Print(printPrefix + "Temporary Error: " + IntToStr(err) + " " + ErrorDescription(err) + ". Waiting.");
			}
         else {
            Print(printPrefix + "Permanent Error: " + IntToStr(err) + " " + ErrorDescription(err) + ". Giving up.");
            return(-1);
         }
      }
      else {
         Print(printPrefix + "Must wait for trade context.");
		}

      Sleep(MathRand() / 10);
   }
   return (-2);
}

/* -----------------------------------------------------------
   OrderCloseReliable
	Drop-in replacement for OrderClose().
	Try to handle all errors and locks and return only if successful
	or if the error can not be handled or waited for.
	simplex: modified logging
   ----------------------------------------------------------- */
bool OrderCloseReliable(	int ticket,
									double lots,
									int _slippage,
									color arrow_color = clrNONE
								) {
   bool success;
   int err;
   double price = 0;
   bool selected = OrderSelect(ticket, SELECT_BY_TICKET, MODE_TRADES);
	string printPrefix = eaLog(__FUNCTION__, ticket2String(ticket));
   
   while (true) {
      if (IsStopped()) {
         Print(printPrefix, " Trading is stopped. Cannot close order!");
         return(false);
      }
      RefreshRates();
      if (OrderType() == OP_BUY) {
         price = MarketInfo(OrderSymbol(), MODE_BID); // close long at bid
      }
      if (OrderType() == OP_SELL) {
         price = MarketInfo(OrderSymbol(), MODE_ASK); // close short at ask
      }
      if (!IsTradeContextBusy()) {
         success = OrderClose(ticket,
                              lots,
                              NormalizeDouble(price, (int)MarketInfo(OrderSymbol(), MODE_DIGITS)),
                              _slippage,
                              arrow_color);
         if (success) {
            Print(printPrefix, "Order closed.");
            return(true); 									// the normal exit
         }

         err = GetLastError();
         if (IsTemporaryError(err)) {
            Print(printPrefix, "Temporary Error: " + IntToStr(err) + " " + ErrorDescription(err) + ". waiting.");
         } 
         else {
            Print(printPrefix, "Permanent Error: " + IntToStr(err) + " " + ErrorDescription(err) + ". giving up.");
            return(false);
         }
      } 
      else {
         Print(printPrefix, "Must wait for trade context");
      }
      Sleep(MathRand() / 10);
   }
   
   return (false);
}

/* -----------------------------------------------------------
   OrderDeleteReliable
	Drop-in replacement for OrderDelete().
	Try to handle all errors and locks and return only if successful
	or if the error can not be handled or waited for.
	simplex: modified logging
   ----------------------------------------------------------- */
bool OrderDeleteReliable(int ticket) {
   bool success;
   int err;
	string printPrefix = eaLog(__FUNCTION__, ticket2String(ticket));
   // Print(printPrefix);
	
   while (true) {
		waitForTradeContext(printPrefix);

      success = OrderDelete(ticket);

      if (success) {
         Print(printPrefix, "Order deleted.");
         return(true);
      }

      err = GetLastError();
      if (IsTemporaryError(err)) {
         Print(printPrefix, "Temporary Error: " + IntToStr(err) + " " + ErrorDescription(err) + ". waiting.");
      } 
      else {
         Print(printPrefix, "Permanent Error: " + IntToStr(err) + " " + ErrorDescription(err) + ". giving up.");
         return(false);
      }
      Sleep(MathRand() / 10);
   }
   return false;
}

void  deletePendingOrders(string _symbb, int _magicNumber, int _orderType)
{
   for (int u = OrdersTotal() - 1; u >= 0; u--)
   {
      if (OrderSelect(u, SELECT_BY_POS, MODE_TRADES))
      {
         if (OrderSymbol() == _symbb && OrderMagicNumber() == _magicNumber && OrderType() == _orderType)
         {
            OrderDeleteReliable(OrderTicket());
         }
      }
   }
}

void  deletePendingOrders(int _magicNumber, int _orderType)
{
   for (int u = OrdersTotal() - 1; u >= 0; u--)
   {
      if (OrderSelect(u, SELECT_BY_POS, MODE_TRADES))
      {
         if (OrderMagicNumber() == _magicNumber && OrderType() == _orderType)
         {
            OrderDeleteReliable(OrderTicket());
         }
      }
   }
}

int getLastOpenOrderTicket(string _symbol, string type, int _magicNbr)
{
   datetime lastTime = 0;
   int _ticket = -1;
   int nbrOfOrders = OrdersTotal();

   for (int u = nbrOfOrders - 1; u >= 0; u--)
   {
      if (OrderSelect(u, SELECT_BY_POS, MODE_TRADES)== true)
      {
         if (type == "buy")
         {
            if (OrderType() == OP_BUY && OrderMagicNumber() == _magicNbr && OrderSymbol() == _symbol)
            {
               if (OrderOpenTime() > lastTime)
               {
                  lastTime = OrderOpenTime();
                  _ticket = OrderTicket();
               }
            }
         }
         if (type == "sell")
         {
            if (OrderType() == OP_SELL && OrderMagicNumber() == _magicNbr && OrderSymbol() == _symbol )
            {
               if (OrderOpenTime() > lastTime)
               {
                  lastTime = OrderOpenTime();
                  _ticket = OrderTicket();
               }
            }
         }
      }
      else
      {
         Print("Error in get last open order (buy or sell)");
      }
   
   }
   return _ticket;
}

void closeAllOrders(int _type, string _symbol, int _magicNbr)
{

   int nbrOfOrders = OrdersTotal();
   
   //string type = StringUpper(_type);
   
   for ( int u = nbrOfOrders-1; u >= 0; u--)
   {
      if (OrderSelect(u, SELECT_BY_POS, MODE_TRADES))
      {
         if (_type == OP_BUY && OrderSymbol() == _symbol && OrderType() == OP_BUY && OrderMagicNumber() == _magicNbr)
         {
            OrderCloseReliable(OrderTicket(), OrderLots(), 10, 0);
         }
         if (_type == OP_SELL && OrderSymbol() == _symbol && OrderType() == OP_SELL && OrderMagicNumber() == _magicNbr)
         {
            OrderCloseReliable(OrderTicket(), OrderLots(), 10, 0);
         }
      }
   }
}

//+------------------------------------------------------------------+
int countOpenTrades(string _CMD, string _symbol, int _magicBuy, int _magicSell)      // count number of open orders for buy, sell or both
{
   int all=0, buy=0, sell=0;
   
   string _command = _CMD; //StringToUpper(_CMD);
   //Alert(_symbol);

   for(int i = OrdersTotal() - 1; i >= 0; i--)
   {
      if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
      {
         if (OrderSymbol() == _symbol && (OrderMagicNumber() == _magicBuy || OrderMagicNumber() == _magicSell))
         {
            if(OrderType() == OP_BUY) 
            {
               buy ++;
               //Alert(_symbol);

            }
            if(OrderType() == OP_SELL) 
            {
               sell ++;
               //Alert(_symbol); 
            }
            all ++;
         }
      }
   }
   //Print(_command);
   if(_command == "ALL") return all;
   if(_command == "buy") return buy;
   if(_command == "sell") return sell;
   
   return -1;
}

/* -----------------------------------------------------------
   StringLeftTrim
   ------------------
   hanover --- extensible functions (np).mqh
   ------------------
   Removes all leading spaces from a string
   Usage:    string x=StringLeftTrim("  XX YY  ")  returns x = "XX  YY  "
   _char defaults to space(s), but may contain a list of characters to be stripped
	----------------------------------------------------------- */
string StringLeftTrim(const string str, const string _char = " ") {
   bool left     = true;
   string outstr = "";

   for (int i = 0; i<StringLen(str); i++) {
      if (StringFind(_char, StringSubstr(str, i, 1)) < 0 || !left) {
         outstr = outstr + StringSubstr(str, i, 1);
         left   = false;
      }
   }
   return(outstr);
}

/* -----------------------------------------------------------
   StringRightTrim
   ------------------
   hanover --- extensible functions (np).mqh
   ------------------
	Removes all trailing spaces from a string
	Usage:    string x=StringRightTrim("  XX YY  ")  returns x = "  XX  YY"
	_char defaults to space(s), but may contain a list of characters to be stripped
	----------------------------------------------------------- */
string StringRightTrim(const string str, const string _char=" ") {
	int pos = 0;
	for(int i=StringLen(str)-1; i>=0; i--) {
		if (StringFind(_char,StringSubstr(str,i,1)) < 0) {
			pos = i;
			break;
		} 
	}
	string outstr = StringSubstr(str,0,pos+1);
	return(outstr);
}




/* -----------------------------------------------------------
   NormalizeLots
   ----------------------------------------------------------- */
double NormalizeLots(string sym, double lots) {
   RefreshRates();
   double step = MarketInfo(sym, MODE_LOTSTEP);
   int norm = 0;
   if (step == 1   ) norm = 0;
   if (step == 0.1 ) norm = 1;
   if (step == 0.01) norm = 2;
   return NormalizeDouble(lots, norm);
}


/* -----------------------------------------------------------
   OrderTypeToStr
	Converts integer value of order command type to its code
	simplex mod: OP_ALL
   ----------------------------------------------------------- */
string OrderTypeToStr(int cmd) {
   switch(cmd) { 
      case OP_BUY: 
         return "OP_BUY"; 
      case OP_SELL: 
         return "OP_SELL";
      case OP_BUYLIMIT: 
         return "OP_BUYLIMIT";
      case OP_SELLLIMIT: 
         return "OP_SELLLIMIT";
      case OP_BUYSTOP: 
         return "OP_BUYSTOP";
      case OP_SELLSTOP: 
         return "OP_SELLSTOP";
      case OP_ALL: 
         return "ALL ORDERS";
   }
   return ("ORDERTYPE UNKNOWN: " + IntegerToString(cmd));
}


/* -----------------------------------------------------------
   IntToStr
   ----------------------------------------------------------- */
string IntToStr(int i) {
   return (IntegerToString(i));
}

/* -----------------------------------------------------------
   IsSpreadOk
	Checks spread
	simplex mod, 2016
   ----------------------------------------------------------- */
bool IsSpreadOk(string _sym, double _maxSpread) 
{
	
	// simplex mod: makes no sense during backtest
	if (IsTesting()) {
      return true;
	}
	if (_maxSpread <= 0.0)
	{
	   return true;
	}
	
   double currentSpread = MathAbs( MarketInfo(_sym, MODE_ASK) - MarketInfo(_sym, MODE_BID)) * _myMultiplier;
   
   if(currentSpread > _maxSpread) {
      Print("Current spread " + DoubleToStr(currentSpread, 1) + " is bigger than max allowed spread " + DoubleToStr(_maxSpread, 1));
      return false;
   }
   else {
      return true;
	}
}

void writeToLogFile( string strLog, string preamble)
{
   fileName = preamble + IntegerToString(Year()) + IntegerToString(Month()) + IntegerToString(Day()) + "_log.log";
   
   int file_handle=FileOpen(fileName,FILE_READ|FILE_WRITE|FILE_TXT);
   if(file_handle != INVALID_HANDLE) 
   {
      FileWriteString(file_handle,strLog);
      FileClose(file_handle);
   }

}

//+------------------------------------------------------------------+
string StringUpper(string str)
//+------------------------------------------------------------------+
// Converts any lowercase characters in a string to uppercase
// Usage:    string x=StringUpper("The Quick Brown Fox")  returns x = "THE QUICK BROWN FOX"
{
  string outstr = "";
  string lower  = "abcdefghijklmnopqrstuvwxyz";
  string upper  = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  for(int i=0; i<StringLen(str); i++)  {
    int t1 = StringFind(lower,StringSubstr(str,i,1),0);
    if (t1 >=0)  
      outstr = outstr + StringSubstr(upper,t1,1);
    else
      outstr = outstr + StringSubstr(str,i,1);
  }
  return(outstr);
} 
};